import { QueryRouterServer } from '@kevisual/router/browser';
import { useContextKey } from '@kevisual/store/config';
type InitFn = () => any;
type InitModule = {
  init?: boolean;
  loaded?: boolean;
  fn?: InitFn;
  key?: string;
};
class UseInit {
  modules = new Map<string, InitModule>();
  callbacks = new Map<string, InitFn>();
  constructor() {}
  async run(fn: InitFn, key: string, force = false) {
    if (this.modules.has(key)) {
      const module = this.modules.get(key)!;
      if (module.init) {
        if (force) {
          await module.fn?.();
        }
        return;
      }
      await module.fn?.();
      module.init = true;
    } else {
      this.modules.set(key, {
        init: true,
        fn,
        key,
      });
      await fn();
    }
  }
}
const module = useContextKey<UseInit>('module', () => {
  return new UseInit();
});
const app = useContextKey<QueryRouterServer>('app');

const loadApps = async () => {
  app
    .route({
      path: 'dev',
      key: 'preview-container',
    })
    .define(async (ctx) => {
      const data = ctx.query?.data || {};
      const code = data.code || '';
      if (!code) {
        ctx.throw?.(400, 'code is required');
      }
      const url = URL.createObjectURL(new Blob([code], { type: 'application/javascript' }));
      console.log('dev route');
      const module = await import(url);
      URL.revokeObjectURL(url);
      if (module?.render) {
        module.render();
      }
    })
    .addTo(app);
};
export const route = async (page?: any) => {
  console.log('home route page', page);
  console.log('home route app', app);
  const div = document.createElement('div');

  div.className = 'text-4xl text-center p-4 bg-blue-200';
  div.innerHTML = 'Browser Apps';
  div.onclick = () => {
    location.href = '/list';
  };
  document.body.appendChild(div);
  module.run(() => {
    loadApps();
  }, 'home');
  const res = await app.call({
    path: 'dev',
    key: 'preview-container',
    payload: {
      data: {
        code: `
        export const render = () => {
          const div = document.createElement('div');
          div.className = 'text-4xl text-center p-4 bg-blue-200';
          div.innerHTML = 'Browser Apps Preview';
          document.body.appendChild(div);
          }`,
      },
    },
  });
  console.log('res', res);
};
