import { StoreManager } from '@kevisual/store';
import type { StateCreator, StoreApi } from 'zustand';
import { useContextKey } from '@kevisual/store/config';
import { postOllama, postOllamaStrem } from '@/response/ollama/ollama';
import { nanoid } from 'nanoid';

export const store = useContextKey('storeManager', () => {
  return new StoreManager();
});

type AiChatMessage = {
  content?: string;
  role?: 'user' | 'ai';
  id?: string;
  response?: Response;
};
type AiChatStore = {
  messages: AiChatMessage[];
  loading: boolean;
  setLoading: (loading: boolean) => void;
  sendMessage: (text: string) => Promise<any>;
  end: boolean;
  setEnd: (end: boolean) => void;
};
export const createAiChatStore: StateCreator<AiChatStore, [], [], AiChatStore> = (set, get) => {
  return {
    messages: [],
    loading: false,
    end: true,
    setEnd: (end) => {
      set({ end });
    },
    setLoading: (loading) => {
      set({ loading });
    },
    sendMessage: async (text) => {
      // const res = await postOllama(text);
      const id = nanoid(8);
      // const getDataFromOllamaJson = (value: typeof res) => {
      //   const id = nanoid(8);
      //   return {
      //     content: value.response as string,
      //     role: 'ai' as const,
      //     id: id,
      //   };
      // };
      // const data = getDataFromOllamaJson(res);
      set({ loading: true });
      const newMessage = [
        ...get().messages,
        { content: text, role: 'user' as const, id: id }, //
      ];
      // set({
      //   messages: newMessage,
      // });
      const resStream = await postOllamaStrem(text);
      set({ loading: false });
      const getDataFromOllamaStream = (value: Response) => {
        const id = nanoid(8);
        return {
          role: 'ai' as const,
          id: id,
          response: value,
        };
      };
      const dataStream = getDataFromOllamaStream(resStream);

      set({
        messages: [...newMessage, dataStream],
        end: false,
      });
    },
  };
};
export const aiChatStore: StoreApi<AiChatStore> = store.createStore(createAiChatStore, 'aiStore');
