type TaskData = {
  element?: HTMLElement;
  onUnload?: () => void;
  onLoad?: () => void;
};
// 定义任务队列
const loadTasks: TaskData[] = [];
const unloadTasks: TaskData[] = [];
let isSchedulerLoadRunning = false; // 调度器状态，加载
let isSchedulerUnLoadRunning = false; // 调度器状态

export const addTask = (task: TaskData) => {
  if (task.onLoad || task.onUnload) {
    loadTasks.push(task);
  }
  if (!isSchedulerLoadRunning) {
    isSchedulerLoadRunning = true;
    requestIdleCallback(processTasks);
  }
};
export const addUnloadTask = (task: TaskData) => {
  if (task.onUnload) {
    unloadTasks.push(task);
  }
};
// 调度函数
const processTasks: IdleRequestCallback = (deadline) => {
  // 优先处理高优先级任务
  while (loadTasks.length && deadline.timeRemaining() > 0) {
    const task = loadTasks.shift();
    if (task?.element?.isConnected) {
      task.onLoad?.();
      if (task.onUnload) {
        addUnloadTask(task);
      }
    } else if (task) {
      addTask(task);
    }
  }
  // 检查是否还有任务
  if (loadTasks.length) {
    requestIdleCallback(processTasks);
  } else {
    // 没有任务，停止调度器
    isSchedulerLoadRunning = false;
  }
};
requestIdleCallback(processTasks);
